// AltHLDemo.java

import java.awt.*;
import java.awt.event.*;

import javax.swing.*;
import javax.swing.border.*;

public class AltHLDemo
{
   public static void main (String [] args)
   {
      // Establish a look and feel for this application's about dialog box.

      setLookAndFeel ();

      // Construct a modal about dialog box with a title, a message, and link
      // information. A frame is created so you can observe the mouse pointer
      // changing to a hand when the mouse pointer is positioned over the
      // hyperlink.

      JFrame f = new JFrame ();
      f.setVisible (true);
      AboutDialog ad = new AboutDialog (f,
                                        "AltHLDemo",
                                        "AltHLDemo 1.0",
                                        "by Jeff Friesen",
                                        "http://www.javajeff.mb.ca");

      // Present the about dialog box to the user. Because the about dialog box 
      // is modal, the thread executing the main() method will not exit this 
      // method until the dialog box is disposed.

      ad.setVisible (true);
   }

   static void setLookAndFeel ()
   {
      try
      {
          // Return the name of the LookAndFeel class that implements the
          // native OS look and feel. If there is no such look and feel, return 
          // the name of the default cross platform LookAndFeel class.

          String slafcn = UIManager.getSystemLookAndFeelClassName ();

          // Set the current look and feel to the look and feel identified by
          // the LookAndFeel class name.

          UIManager.setLookAndFeel (slafcn);
      }
      catch(Exception e)
      {
      }
   }
}

class AboutDialog extends JDialog
{
   AboutDialog (JFrame parent, String title, String message, String linkText,
                String link)
   {
      // Assign title to the dialog box's title bar and make sure dialog box is 
      // modal.

      super (parent, title, true);

      // Specify the root pane as opaque so that its entire content pane is
      // painted to remove damage from tooltips and so on. This is only needed
      // in J2SE 1.4.x (and probably previous versions as well).

      getRootPane ().setOpaque (true);

      // Tell dialog box to automatically hide and dispose itself when the user 
      // selects the Close menu item from the dialog box's system menu.

      setDefaultCloseOperation (DISPOSE_ON_CLOSE);

      // Establish the dialog box's border area. A compound border establishes
      // a beveled outer border with an empty inner border, which is used as a
      // margin.

      Border border = BorderFactory.createBevelBorder (BevelBorder.LOWERED);
      Border margin = BorderFactory.createEmptyBorder (10, 10, 10, 10);
      Border cborder = BorderFactory.createCompoundBorder (border, margin);
      getRootPane ().setBorder (cborder);

      // Use a vertical Box as the dialog box's content pane, to simplify
      // layout.

      Box box = Box.createVerticalBox ();
      setContentPane (box);

      // Add a message (typically identifying the program) label to the box's
      // top. This label is horizontally centered within the box.

      JLabel label = new JLabel (message);
      label.setAlignmentX (0.5f);
      box.add (label);

      // Add a 30-pixel vertical blank space below the message label to the box 
      // (for aesthetic purposes).

      box.add (Box.createVerticalStrut (30));

      // Add a hyperlink to the box's middle. This hyperlink is horizontally
      // centered within the box.

      Hyperlink hyperlink;
      hyperlink = new Hyperlink (new LinkAction (linkText, link));
      hyperlink.setAlignmentX (0.5f);
      box.add (hyperlink);

      // Add a 30-pixel vertical blank space below the hyperlink to the box
      // (for aesthetic purposes).

      box.add (Box.createVerticalStrut (30));

      // Add an OK button to the box's bottom. This button is horizontally
      // centered within the box. Assign an action listener to hide and dispose 
      // the dialog box (when the button is clicked).

      JButton button = new JButton ("OK");
      button.setAlignmentX (0.5f);
      button.addActionListener (new ActionListener ()
                                {
                                    public void actionPerformed (ActionEvent e)
                                    {
                                       dispose ();
                                    }
                                });
      box.add (button);

      // Size dialog box to fit the preferred size and layouts of its
      // components.

      pack ();

      // Center the dialog box (when displayed) relative to its parent window.
      // If the parent window is not showing, the dialog box is centered on the 
      // screen.

      setLocationRelativeTo (parent);

      // Give input focus to the button.

      button.requestFocusInWindow ();
   }

   private class LinkAction extends AbstractAction
   {
      LinkAction (String linkText, String link)
      {
         // Save the link's text and the actual link for later recall when the
         // user clicks the hyperlink.

         putValue (Action.NAME, linkText);
         putValue (Action.SHORT_DESCRIPTION, link);
      }

      public void actionPerformed (ActionEvent e)
      {
         // Retrieve the actual link and output link to the console (for test
         // purposes).

         String link = (String) getValue (Action.SHORT_DESCRIPTION);
         System.out.println (link);

         // Launch the default Web browser and have the Web browser display the 
         // Web page associated with the link.

         BareBonesBrowserLaunch.openURL (link);
      }
   }
}

class Hyperlink extends JButton
{
   // Hyperlink color.

   private static final Color LINK_COLOR = Color.blue;

   // This border erases the single underline from the HOVER_BORDER.

   private static final Border LINK_BORDER =
     BorderFactory.createEmptyBorder (0, 0, 1, 0);

   // This border presents a single underline in the LINK_COLOR.

   private static final Border HOVER_BORDER =
     BorderFactory.createMatteBorder (0, 0, 1, 0, LINK_COLOR);

   Hyperlink (final Action action)
   {
      // Replace the default button border with a LINK_BORDER. This border
      // results in no border being displayed. (Borders give buttons their
      // distinctive appearance.)

      setBorder (LINK_BORDER);

      // Do not paint button background in J2SE 5.0's new Metal Look and Feel.
      // This also applies to Windows XP and other look and feels.

      setContentAreaFilled (false);

      // If this component has a JFrame ancestor, change the mouse pointer to a 
      // hand when it is positioned over the button's text.

      setCursor (Cursor.getPredefinedCursor (Cursor.HAND_CURSOR));

      // Do not paint a focus rectangle (which will not have the correct size
      // anyway) on the button when the button has the focus.

      setFocusPainted (false);

      // The button's text appears in the color specified by LINK_COLOR.

      setForeground (LINK_COLOR);

      // Establish an action listener to invoke the action's actionPerformed()
      // method when the user selects the button (or we can think about this as 
      // the user selecting a hyperlink).

      addActionListener (new ActionListener ()
                         {
                             public void actionPerformed (ActionEvent e)
                             {
                                action.actionPerformed (e);
                             }
                         });

      // Install a focus listener that underlines the button text when the
      // button receives focus and removes this underline when focus is lost.

      addFocusListener (new LinkFocusListener ());

      // Install a mouse listener that causes focus to shift to the button when 
      // the mouse enters the button.

      addMouseListener (new LinkMouseListener ());

      // Obtain the contents of the action's NAME property as the button's
      // text. This is the link's text. 

      setText ((String) action.getValue (Action.NAME));

      // Obtain the contents of the action's SHORT_DESCRIPTION property as the 
      // button's tooltip text. This is the link.

      setToolTipText ((String) action.getValue (Action.SHORT_DESCRIPTION));
   }

   private class LinkFocusListener extends FocusAdapter
   {
      public void focusGained (FocusEvent e)
      {
         ((JComponent) e.getComponent ()).setBorder (HOVER_BORDER); 
      }

      public void focusLost (FocusEvent e)
      {
         ((JComponent) e.getComponent ()).setBorder (LINK_BORDER); 
      }
   }

   private class LinkMouseListener extends MouseAdapter
   {
      public void mouseEntered (MouseEvent e)
      {
         ((JComponent) e.getComponent ()).requestFocusInWindow ();
      } 
   }
}
